#!/usr/bin/env python

# $Id: ducky.py,v 1.12 2005/07/27 19:47:02 jose Exp $

__copyright__ = """
Copyright (c) 2005 Jose Nazario

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import codecs, sys
import DuckyLib
from DuckyLib import technorati as tec
from DuckyLib.yahoo.search import webservices
from DuckyLib import amazon, feedster, gataga, google, noaa, topix, yahoo, ynews
from DuckyLib import delicious, indeed, citeulike, connotea, simplyhired
from DuckyLib import yahoojobs, msn, daypop, blogdigger, flickr
from pprint import pprint

__version__ = '0.9.2'

def info():
    print 'ducky, the web services command line.'
    print 'Version: %s' % __version__
    print 'usage: <CMD> <KEYWORDS>\n'
    print 'Copyright (c) 2005 Jose Nazario, all rights reserved.'
    print 'Uses libraries with these copyrights:'
    print '  Technorati support:', tec.__copyright__
    print '  Google support:', google.__copyright__
    print '  Amazon support:', amazon.__copyright__
    print '  Feedster support:', feedster.__copyright__
    print '  Yahoo! support:', yahoo.search.webservices.__copyright__
    print '  Flickr support:', flickr.__copyright__
    print '  Topix support:', topix.__copyright__
    print '  Yahoo! News support:', ynews.__copyright__
    print '  Yahoo! Jobs support:', yahoojobs.__copyright__
    print '  NOAA support:', noaa.__copyright__
    print '  Gataga support:', gataga.__copyright__
    print '  delicious support:', delicious.__copyright__
    print '  Indeed support:', indeed.__copyright__
    print '  SimplyHired support:', simplyhired.__copyright__
    print '  CiteULike support:', citeulike.__copyright__
    print '  Connotea support:', connotea.__copyright__
    print '  MSN support:', msn.__copyright__
    print '  Daypop support:', daypop.__copyright__
    print '  Blogdigger support:', blogdigger.__copyright__
    sys.exit(0)

def _descape(input):
    """opposite of cgi.escape() ..."""
    input = input.replace('&gt;', '>')
    input = input.replace('&lt;', '<')
    input = input.replace('&quot;', '"')
    input = input.replace('&#39;', """'""")
    input = input.replace('&amp;', '&')
    return input

def noKey(name):
    print 'Support for %s disabled, no license key found.' % name

def err():
    print 'An error occured.'
    sys.exit(-1)

def setKeys():
    try: amazon.setLicense(amazon.getLicense())
    except: noKey('amazon')
    try: google.setLicense(google.getLicense())
    except: noKey('google')
    try: tec.setLicense(tec.findkey())
    except: noKey('technorati')
    try: flickr.setLicense(flickr.getLicense())
    except: noKey('flickr')

def doYNews(terms):
    """get Yahoo! news results via RSS"""
    try: results = ynews.search('+'.join(terms))[1:9] # skip header, footer
    except: err()
    i = 1
    for r in results:
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)' % r['source'],
        except: pass
        print '\n   %s' % r[u'description']
        print '   %s' % r[u'pubDate']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doYJobs(terms):
    """search Yahoo's job search engine"""
    try: results = yahoojobs.search('+'.join(terms))
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        print '\n   %s' % r[u'pubDate']
        print '   %s' % r[u'description']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doNoaa(site):
    """get NOAA weather results from an airport"""
    try: results = noaa.search(site.upper())
    except: err()
    for r in results:
	print 'Current conditions:\n%s' % r[u'title']
	print '%s' % r[u'description'].strip()
	print '%s' % r[u'link']
    sys.exit(0)

def doTopix(terms):
    """get topix results via RSS"""
    try: results = topix.search('+'.join(terms))[1:9] # skip header, footer
    except: err()
    i = 1
    for r in results:
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)' % r['source'],
        except: pass
        print '\n   %s' % r[u'description']
        print '   %s' % r[u'pubDate']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doAmazon(type, terms):
    """wrap the amazon search interface"""
    try: 
        if type == 'keyword': books = amazon.searchByKeyword('%s' % ' '.join(terms))
        elif type == 'asin': books = amazon.searchByASIN('%s' % ' '.join(terms))
    except: err()
    for b in books: 
        print '\n%s: %s  ' % (b.Catalog, b.ProductName),
        try: print 'Price: %s' % b.OurPrice,
        except: pass
        try: 
            author = b.Authors.Author
            if len(author) < 5: print '\nAuthors: %s' % ', '.join(author),
            else: print '\nAuthor: %s' % author,
        except: pass
        print '\n%s' % b.URL
    sys.exit(0)

def doTec(terms):
    """wrap the technorati search interface"""
    try: data = tec.search('%s' % ' '.join(terms))
    except: err()
    i = 1
    for r in data['search']:
        r['excerpt'] = r['excerpt'].replace('<strong class="keyword">', '')
        r['excerpt'] = r['excerpt'].replace('</strong>', '')
        blog = r['weblog']
        try:
            print '\n%d. %s' % (i, _descape(blog['name'].encode('utf-8')))
            print '     %s' % _descape(r['excerpt'].encode('utf-8'))
            print u'     %s' % blog['url']
            print u'     %s' % blog['lastupdate']
        except: pass
        i = i + 1
    sys.exit(0)

def doBlogdig(terms):
    """search the Daypop search engine"""
    try: results = blogdigger.search('+'.join(terms))
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        print '\n   %s' % r[u'pubDate']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doDaypop(terms):
    """search the Daypop search engine"""
    try: results = daypop.search('%20'.join(terms))
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        print '\n   %s' % r[u'description']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doMsn(terms):
    """search the MSN Beta search engine"""
    try: results = msn.search('%20'.join(terms))
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        print '\n   %s' % r[u'description']
        print '\n   %s' % r[u'pubDate']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doSimply(terms):
    """search the Simply Hired job search engine"""
    try: results = simplyhired.search('_'.join(terms))
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        print '\n   %s' % r[u'pubDate']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doIndeed(terms):
    """search indeed job search engine"""
    try: results = indeed.search('+'.join(terms))
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)' % r['dc:creator'],
        except: pass
        print '\n   %s' % r[u'pubDate']
        print '   %s' % _descape(r[u'description']).replace('<br>', '')
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doFlickr(term):
    """search del.icio.us by tag"""
    import time
    try: results = flickr.photos_search(tags=term)
    except: err()

    def untag(line):
        line = line.replace("""<a href="http://www.geobloggers.com">GeoTagged</a>""", '')
        line = line.replace("""<a href="http://www.geobloggers.com" rel="nofollow">GeoTagged</a>""", '')
        return line.strip()

    # pprint(flickr.photos_search(tags=term))
    i = 1
    for r in results[:10]:
        if not r.ispublic: next
        o = r.owner
        print '%d. %s (%s) %s' % (i, o.realname, o.username, o.location)
        try: print '   %s' % r.getURL()
        except: pass
        if len(r.description) > 1: 
            descr = untag(r.description)
            title = untag(r.title)
            print '   %s: %s' % (title, _descape(descr))
        print '   %s' % time.strftime('%B %d, %Y %H:%M', time.localtime(float(r.dateposted)))
        i = i + 1
    sys.exit(0)

def doDel(term):
    """search del.icio.us by tag"""
    try: results = delicious.search(term)
    except: err()
    i = 1
    for r in results[:10]:
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)' % r['dc:creator'],
        except: pass
        print '\n   %s' % r[u'dc:date']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doCiteulike(terms):
    """wrap the CIteULike search engine"""
    try: results = citeulike.search('%s' % '%20'.join(terms))
    except: err()
    i = 1
    for r in results:
        if r[u'title'] == 'untitled': continue
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)\n' % r['dc:creator'],
        except: pass
        if r.has_key(u'dc:source'): print '   %s' % r[u'dc:source']
        if r.has_key(u'dc:identifier'): print '   %s' % r[u'dc:identifier']
        print '   %s' % r[u'link']
        if i == 10: break
        i = i + 1
    sys.exit(0)

def doConnotea(terms):
    """wrap the Connotea search engine"""
    try: results = connotea.search('%s' % '/'.join(terms))
    except: err()
    i = 1
    for r in results:
        if r[u'title'] == 'untitled': continue
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)\n' % r['dc:creator'],
        except: pass
        if r.has_key(u'dc:source'): print '   %s' % r[u'dc:source']
        if r.has_key(u'dc:identifier'): print '   %s' % r[u'dc:identifier']
        print '   %s' % r[u'link']
        if i == 10: break
        i = i + 1
    sys.exit(0)

def doTag(terms):
    """wrap the gataga search engine"""
    try: results = gataga.search('%s' % '%20'.join(terms))
    except: err()
    i = 1
    for r in results:
        print '\n%d. %s' % (i, r[u'title']),
        try: print ' (%s)' % r['dc:creator'],
        except: pass
        print '\n   %s' % r[u'dc:date']
        print '   %s' % r[u'link']
        i = i + 1
    sys.exit(0)

def doFeed(terms):
    """wrap the feedster search engine ..."""
    try: results = feedster.search('%s' % '+'.join(terms))
    except: err()
    i = 1
    for r in results:
        try: print '\n%d. %s' % (i, r[u'title'].strip().encode('utf-8')),
        except: pass
        try: print ' (%s)' % r['source'].strip().encode('utf-8'),
        except: pass
        print '\n   %s' % r[u'pubDate'].strip()
        print '   %s' % r[u'link'].strip()
        i = i + 1
    sys.exit(0)

def doYahoo(terms):
    """wrap the Yahoo! WS search ..."""
    try: x = webservices.create_search("web", "YahooDemo", query=' '.join(terms), results=10)
    except: err()
    srch = x.parse_results()
    i = 1
    for s in srch._get_results():
       print '\n%d. %s' % (i, s['Title'])
       print '   %s' % s['Summary']
       print '   %s' % s['Url']
       i = i + 1
    sys.exit(0)

def doGoogle(terms):
    """wrap the google search interface"""
    try: data = google.doGoogleSearch('%s' % ' '.join(terms))
    except: err()
    print 'Results 1 - 10 of about %s for %s' % \
        (str(data.meta.estimatedTotalResultsCount), ' '.join(terms))
    i = 1
    for r in data.results: 
       title = r.title
       title = title.replace('<b>', '')
       title = title.replace('</b>', '')
       snippet = r.snippet
       snippet = snippet.replace('<b>', '')
       snippet = snippet.replace('</b>', '')
       snippet = snippet.replace('<br>', '')
       url = r.URL
       url = url.replace('<b>', '')
       url = url.replace('</b>', '')
       print u'%d. %s' % (i, title)
       print u'     %s' % _descape(snippet)
       print u'     %s\n' % url
       i = i + 1
    sys.exit(0)

def main():
    setKeys()
    sys.stdout = codecs.lookup('utf-8')[-1](sys.stdout)

    argv0 = sys.argv[0].split('/')[-1]
    if len(sys.argv) == 1: info()

    # helper
    def doAsin(terms): doAmazon('asin', terms)

    # these take sys.argv[1:] as their args.
    # key: 	cmdline invocation
    # value: 	eval function (as _map[key](sys.argv[1:])
    _map = {'asin':	doAsin,
            'bdig':	doBlogdig,
            'cite':	doCiteulike,
            'conn':	doConnotea,
            'day':	doDaypop,
            'feed':	doFeed,
            'g': 	doGoogle,
            'ind':	doIndeed,
            'isbn':	doAsin,
            'msn':	doMsn,
            'simp':	doSimply,
            'tag':	doTag,
            'tec':	doTec,
            'topix':	doTopix,
            'ya':	doYahoo,
            'ynews':	doYNews,
            'yjobs':	doYJobs
           }

    # amazon mappings to locales
    # sets the locale, then does the search
    _ammap = {'am':	'us',
              'amuk':	'uk',
              'ameu':	'de',
              'amjp':	'jp'
	     }

    # these are functions that take only sys.argv[1]
    # key:	cmdline invocation
    # value:	function (as _1map[argv0](sys.argv[1])
    _1map = {'del':	doDel,
             'fl':	doFlickr,
             'noaa':	doNoaa
            }

    if _map.has_key(argv0): _map[argv0](sys.argv[1:])
    elif _ammap.has_key(argv0):
        amazon.setLocale(_ammap[argv0])
        doAmazon('keyword', sys.argv[1:])
    elif _1map.has_key(argv0): _1map[argv0](sys.argv[1])
    else: err()

if __name__ == '__main__':
    main()
